#include "acquisizione_tags_svg.h"


static void identifica_tipo( TagSvg *t, char *r ) {
    int i;

    for( i=0; i<kTotTipi; ++i ) {
        char *p = strstr( r, kSTipi[i] );
        if( NULL!=p ) break;
    }

    t->tipo = i;
}


static void identifica_classe( TagSvg *t, char *r ) {
    int i;

    for( i=0; i<kTotClassi; ++i ) {
        char cTmp[32] = "\"";
        size_t l = strlen( kSClassi[i] );
        char *p;
        memcpy( cTmp+1, kSClassi[i], l );
        cTmp[++l] = '\"';
        p = strstr( r, cTmp );
        if( NULL!=p ) break;
    }

    t->classe = i;
}


static void identifica_attributo( char **pp, char *tag, const char *attrbt ) {
    char *p = strstr( tag, attrbt );
    if( p ) p += strlen( attrbt );
    *pp = p;
}


static void elimina_virgolette( char *s ) {
    while( *s ) { if('\"'==*s ) *s = '\0'; ++s; }
}


static int tochenizza_tag_svg( TagSvg *t, char *r ) {
    if( NULL==t || NULL==r ) return 0;

    t->_tag = r; /* memorizza il puntatore alla stringa dei dati originale */
    identifica_tipo( t, r );
    identifica_classe( t, r );
    identifica_attributo( &t->_fill, r, kSAttr[kFill] );
    identifica_attributo( &t->_stroke, r, kSAttr[kStroke] );
    identifica_attributo( &t->_stroke_width, r, kSAttr[kStroke_width] );
    identifica_attributo( &t->_stroke_linecap, r, kSAttr[kStroke_linecap] );
    identifica_attributo( &t->_stroke_linejoin, r, kSAttr[kStroke_linejoin] );
    identifica_attributo( &t->_fill_rule, r, kSAttr[kFill_rule] );
    identifica_attributo( &t->_transform, r, kSAttr[kTransform] );
    identifica_attributo( &t->_points, r, kSAttr[kPoints] );
    identifica_attributo( &t->_d, r, kSAttr[kD] );
    elimina_virgolette( r );

    return 1;
}


static int confronta_per_tipo( const void *p1, const void *p2 ) {
    int t1 = ((TagSvg*)p1)->tipo;
    int t2 = ((TagSvg*)p2)->tipo;
    return t1>t2 ? 1 : (t1<t2?-1:0);
}


static void riordina_per_tipo( TagSvg *t, size_t qt ) {
    qsort( t, qt, sizeof(*t), confronta_per_tipo );
}


static int confronta_per_classe( const void *p1, const void *p2 ) {
    int c1 = ((TagSvg*)p1)->classe;
    int c2 = ((TagSvg*)p2)->classe;
    return c1>c2 ? 1 : (c1<c2?-1:0);
}


static void riordina_per_classe( TagSvg *t, size_t qt ) {
    qsort( t, qt, sizeof(*t), confronta_per_classe );
}


#define POLYLINE     (0==strncmp("<polyline ",s->r[i],10))
#define PATH         (0==strncmp("<path ",s->r[i],6))
#define NOT_POLYLINE (0!=strncmp("<polyline ",s->r[i],10))
#define NOT_PATH     (0!=strncmp("<path ",s->r[i],6))


int acquisisci_tags_svg( const char *nome_file, SintesiSvg *s ) {
    size_t qTipi[kTotTipi]={0};
    size_t p, u;  /* primo e ultimo */
    size_t i, j;

    int ok = nome_file&&s;

    if( ok ) {
        memset( s, 0, sizeof(*s) );
        s->r = (char**) T1_file2array( &s->qr, nome_file, '\n' );
        ok = NULL!=s->r;
    }

    if( ok ) {
        s->dim_dati_orig = 0;

        for( i=0; i<s->qr; ++i )
            s->dim_dati_orig += strlen( s->r[i] ) + 2;
    }

    if( ok ) {
        for( i=0; i<s->qr&&(NOT_POLYLINE&&NOT_PATH); ++i );
        for( p=i; i<s->qr&&(POLYLINE||PATH); ++i );

        s->qt = (u=i)-p;
        s->t = calloc( s->qt, sizeof(TagSvg) );

        ok = NULL!=s->t;
    }

    if( ok ) {
        for( i=p, j=0; i<u; ++i, ++j )
            tochenizza_tag_svg( s->t+j, s->r[i] );

        riordina_per_tipo( s->t, s->qt );

        for( j=0, i=0; i<kTotTipi; ++i )
            for( ; j<s->qt&&i==s->t[j].tipo; ++j )
                qTipi[i]++;

        riordina_per_classe( s->t, qTipi[kPolyline] );
        riordina_per_classe( s->t+qTipi[kPolyline], qTipi[kPath] );
    }

    if( !ok )
        distruggi_tags_svg( s );
    return ok;
}

void distruggi_tags_svg( SintesiSvg *s ) {
    if( s ) {
        if( s->r ) free( s->r );
        if( s->t ) free( s->t );
        memset( s, 0, sizeof(*s) );
    }
}
