/**=============================================================================
                          LIBRERIA "TUTTO IN UNO" (T1)
                               di Aldo Carpanelli
                               v1.0.0, 25/10/2023
==============================================================================*/

#ifndef LIBRERIA_TUTTO_IN_UNO_H_INCLUDED
#define LIBRERIA_TUTTO_IN_UNO_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>
#include <commdlg.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <locale.h>


#ifndef ARGB
typedef COLORREF ARGB;
#endif


typedef struct {
    // il riquadro che contiene l'intera area
    INT x, y, w, h;
    // il raggio di arrotondamento degli spigoli
    INT arrotondamento_orizzontale, arrotondamento_verticale;
    // i margini tra bordo e testo
    INT margine_sinistro, margine_destro, margine_superiore, margine_inferiore;
    // l'allineamento del testo
    INT allineamento_orizzontale, allineamento_verticale;
    // il testo da tracciare
    LPCSTR testo;
} T1_AreaTesto_struct;


#define T1_KEYDOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define T1_KEYUP(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)


/*==============================================================================
La funzione T1_anno_bisestile() restituisce 1 o 0, se l'anno e' o non e'
bisestile.
==============================================================================*/

int T1_anno_bisestile( int anno );


/*==============================================================================
I parametri opzione_a e opzione_b devono corrispondere ai valori di due
caratteri visualizzabili ed inseribili come input in console.

Restituisce opzione_a o opzione_b, a seconda di quale tra le opzioni viene
scelta in console.

Non ammette l'immissione di risposte non univoche (o esattamente opzione_a, o
esattamente opzione_b). In caso di risposte non univocamente accettabili
insiste ostinatamente nel chiedere l'immissione di una risposta valida.
==============================================================================*/

int T1_alternativa( const char *msg, int opzione_a, int opzione_b );


/*==============================================================================
Si comporta come RemoveAlpha(), ma restituisce sotto forma di COLORREF il colore
risultante dalla miscelazione tra colore e sfondo.

NOTA: da' per scontato che sfondo sia un colore completamente opaco (l'eventuale
      trasparenza non viene neppure presa in considerazione).
==============================================================================*/

COLORREF T1_ARGB2COLORREF( ARGB colore, ARGB sfondo );


/*==============================================================================
Presenta il messaggio passato tramite il parametro msg, si pone in attesa di una
qualsiasi immissione da parte dell'utente. In uscita, il buffer di stdin e'
vuoto.
==============================================================================*/

void T1_attendi_invio( const char *msg );


/*==============================================================================
Converte in maiuscolo o in minuscolo lt caratteri della stringa C t, secondo il
valore del parametro dir (dir>0, maiucolo; dir<0, minuscolo; dir==0, niente).
Se lt e' ((DWORD)-1), viene convertita l'intera stringa C t.
Se la funzione va a buon fine, restituisce TRUE, altrimenti restituisce FALSE.
La funzione fallisce e restituisce FALSE in due situazioni:
  - se t e' NULL
  - se CharUpperBuffA() o CharLowerBuffA() fallisce a sua volta.
==============================================================================*/

BOOL T1_CambiaCapitalizzazione( LPSTR t, DWORD lt, INT dir );


/*==============================================================================
Analizza la stringa sIn e colloca in sOut l'elenco dei caratteri univoci che
compaiono in essa, nello stesso ordine in cui si presentano la prima volta.

Esempio: sIn  => "Cane non mangia cane"
         sOut => "Cane omgic"

La funzione non fa nulla se...
  - sIn e' NULL
  - sOut e' NULL
  - sIn e sOut sono entrambi NULL

Se tutto va bene restituisce 1 (true), altrimenti restituisce 0 (false).

NOTA: lo spazio di memoria puntato da sOut dev'essere almeno 256 byte;
      la capacita' di sOut non viene viene verificata dalla funzione.
==============================================================================*/

int T1_caratteri_univoci_da_stringa( const char *sIn, char *sOut );


/*==============================================================================
A partire dalla posizione corrente, analizza il testo contenuto nel file fIn e
colloca in sOut l'elenco dei caratteri univoci che compaiono in esso, nello
stesso ordine in cui si presentano la prima volta nel file.

Esempio: fIn  => "Cane non mangia cane"
         sOut => "Cane omgic"

In uscita, fIn e' nella posizione in cui si trovava al momento della chiamata.

La funzione non fa nulla se...
  - fIn e' NULL
  - sOut e' NULL
  - fIn e sOut sono entrambi NULL
  - non e' possibile leggere da fIn

Se tutto va bene restituisce 1 (true), altrimenti restituisce 0 (false).

NOTA: lo spazio di memoria puntato da sOut dev'essere almeno 256 byte;
      la capacita' di sOut non viene viene verificata dalla funzione.
==============================================================================*/

int T1_caratteri_univoci_da_file( FILE *fIn, char *sOut );


/*==============================================================================
Carica come stringa C il testo contenuto nel file nome_file in uno spazio di
memoria dinamica allocato dinamicamente tramite malloc()/realloc() e ne
restituisce il puntatore.
Se l_txt non e' NULL, colloca in *l_txt la lunghezza della stringa C contenuta
nello spazio di memoria allocato e puntato dal valore di ritorno.
Spetta al chiamante liberare lo spazio di memoria dinamica eventualmente
allocato.
In caso di errore restituisce NULL e non alloca alcuna memoria.

NOTA: la funzione fallisce se...
  - non e' possibile accedere al file tramite fopen( "nome_file", "r" )
  - non e' possibile allocare la memoria necessaria
  - le dimensioni del file di testo da caricare superano quelle
    rilevabili tramite ftell()
==============================================================================*/

char *T1_carica_file_txt( const char *nome_file, long *l_txt );


/*==============================================================================
Con fgets() e un buffer da 256 caratteri rileva una stringa da stdin.
Se la stringa inserita non supera le dimensioni del buffer, usa strtoll() per
convertirla in un valore di tipo long long, che viene restituito come valore di
ritorno.

In caso di errore, invia a stdout messaggi che descrivono cosa e' andato storto,
quindi chiede un nuovo inserimento. La funzione non ritorna fino a quando
l'acquisizione del valore long long non ha avuto successo.
==============================================================================*/

long long T1_chiedi_intero_da_console( void );


/*==============================================================================
Riceve tramite il parametro array un puntatore ad un array di puntatori di tipo
qualsiasi che contiene la quantita' di elementi indicata dal parametro qEl.
Compatta tutti i puntatori non NULL in testa all'array, spostando in coda tutti
i puntatori NULL.

Occorre fare molta attenzione al dato da passare alla funzione tramite il
parametro array, assicurandosi che sia effettivamente un array di puntatori come
ad esempio...

void esempio( void ) {
    int i, numeri[10];
    int *array[10];
    size_t non_nulli;

    for( i=0; i<10; ++i ) {
        numeri[i] = i+1;
        array[i] = &numeri[i];
    }

    // annulla alcuni puntatori
    array[0] = NULL;
    array[2] = NULL;
    array[4] = NULL;
    array[5] = NULL;
    array[9] = NULL;

    non_nulli = compatta_array_di_puntatori( array, 10 );
}

In caso di successo restituisce la quantita' dei puntatori non NULL presenti
nell'array. In caso di insuccesso restituisce SIZE_MAX. L'unica causa di
insuccesso segnalata e' il passaggio di un parametro array nullo.
==============================================================================*/

size_t T1_accoda_puntatori_nulli( void *array, size_t qEl );


/*==============================================================================
Inserisce in *sPt le coordinate a schermo corrispondenti alle coordinate locali
*cPt. I puntatori cPt e sPt possono puntare alla stessa struttura POINT.
In  caso di successo viene restituito TRUE, mentre in caso di fallimento viene
restituito FALSE.
La funzione fallisce e restituisce FALSE in tre situazioni:
  - se anche uno solo dei parametri e' NULL;
  - se hwnd non identifica una finestra valida di Win32;
  - se la funzione di Win32 ClientToScreen() fallisce a sua volta.
==============================================================================*/

BOOL T1_ClientPoint2ScreenPoint( HWND hwnd, CONST LPPOINT cPt, LPPOINT sPt );


/*==============================================================================
Inserisce in *cRect la versione locale del RECT riferito allo schermo *sRect.
I puntatori sRect e cRect possono puntare alla stessa struttura RECT.
In  caso di successo viene restituito TRUE, mentre in caso di fallimento viene
restituito FALSE.
La funzione fallisce e restituisce FALSE in tre situazioni:
  - se anche uno solo dei parametri e' NULL;
  - se hwnd non identifica una finestra valida di Win32;
  - se la funzione di Win32 ClientToScreen() fallisce a sua volta.
==============================================================================*/

BOOL T1_ClientRect2ScreenRect( HWND hwnd, LPCRECT cRect, LPRECT sRect );


/*==============================================================================
Riceve un colore espresso come COLORREF e lo restituisce in formato ARGB.
Dal momento che un COLORREF rappresenta sempre un colore del tutto opaco, il
canale alpha del colore restituito come ARGB e' sempre 0xFF.
==============================================================================*/

ARGB T1_COLORREF2ARGB( COLORREF colore );


/*==============================================================================
Riceve le quattro componenti alpha, red, green e blue di un colore e le
restituisce assemblate sotto forma di valore ARGB.
==============================================================================*/

ARGB T1_ComponiARGB( BYTE a, BYTE r, BYTE g, BYTE b );


/*==============================================================================
Riceve le tre componenti red, green e blue di un colore e le restituisce
assemblate sotto forma di valore COLORREF.
==============================================================================*/

COLORREF T1_ComponiCOLORREF( BYTE r, BYTE g, BYTE b );


/*==============================================================================
Riceve in str una stringa c che si suppone rappresenti una frazione e ne
effettua internamente una copia nella quale sostituisce tutti i caratteri ','
con caratteri '.',
Applica strtod() alla copia di str per ricavarne il numeratore.
Salta gli eventuali spazi presenti dopo il numeratore.
Verifica che sia presente l'operatore '/'.
Applica strtod() a cio' che segue l'operatore '/' per ricavarne il denominatore.
Verifica che non esistano caratteri "spuri" dopo il denominatore,
Se numeratore e denominatore sono stati "estratti" correttamente, colloca in *d
l'esito dell'operazione numeratore/denominatore.

In caso di successo delle operazioni restituisce 1 (come per "true").
In caso di insuccesso restituisce 0 (come per "false").

La funzione fallisce se:
  - str e' NULL o d e' NULL
  - risulta impossibile effettuare la copia temporanea di str
  - strtod() fallisce nel rilevare il numeratore o il denominatore
  - l'operatore '/' e' assente o non e' collocato correttamente
  - str contiene caratteri "spuri" estranei alla frazione da convertire
  - il denominatore estratto e' 0.0
==============================================================================*/

int T1_converti_frazione_in_double( const char *str, double *d );


/*==============================================================================
Con la funzione T1_trova_elemento() individua l'elemento richiesto nell'ambito
della stringa C elementi (che si suppone contenga una lista di sottostringhe
separate dal carattere indicato tramite il parametro separatore), quindi ne
effettua una copia in uno spazio di memoria dinamica appositamente allocato con
malloc().
Se la funzione ha successo restituisce il puntatore allo spazio di memoria
dinamica nel quale e' stato copiato l'elemento richiesto (spetta al chiamante
liberare la memoria allocata con free()).
Se la funzione fallisce restituisce NULL.
==============================================================================*/

char *T1_copia_elemento( const char *elementi, char separatore, int indice );


/*==============================================================================
Alloca dinamicamente spazio in memoria per una matrice di qr*qc elementi di
dimensioni dim_el byte ciascuno. L'area della memoria allocata riservata ai dati
viene inizializzata azzerandola.
In caso di successo, restituisce un void* che deve essere assegnato ad un
puntatore a doppia indirezione di tipo corrispondente al tipo di dati che si
intende immagazzinare nella matrice appena creata.
La matrice puo' essere regolarmente impiegata con i classici operatori di
indicizzazione (matrice[riga][colonna]).
La memoria allocata deve essere liberata dal chiamante tramite free().
In caso di errore restituisce NULL e nessuna memoria risulta allocata.

PARAMETRI

    qr      quantita' di righe nella matrice
    qc      quantita' di colonne nella matrice
    dim_el  dimensioni (in byte) di un elemento della matrice

VALORE DI RITORNO

    un puntatore generico allo spazio di memoria allocato dinamicamente e
    in grado di contenere la matrice

ESEMPIO

int main() {
    const size_t qr = 6;
    const size_t qc = 4;

    // crea una matrice di 6x4 elementi in
    // grado di contenere valori di tipo int
    int **m = crea_matrice( qr, qc, sizeof(int) );

    if( m ) {
        size_t r, c;

        // popola la matrice
        for( r=0; r<qr; ++r )
            for( c=0; c<qc; ++c )
                m[r][c] = 10*r+c;

        // mostra il contenuto della matrice
        for( r=0; r<qr; ++r )
            for( c=0; c<qc; ++c )
                printf( "%d%c", m[r][c], c!=qc-1?' ':'\n' );

        // libera la memoria allocata
        free( m );
        m = NULL;
    }

    return 0;
}

==============================================================================*/

void *T1_crea_array_2d( size_t qr, size_t qc, size_t dim_el );


/*==============================================================================
Crea in un unico blocco di memoria dinamica un array a tre dimensioni di w*h*d
celle. I dati veri e propri sono preceduti da una serie di puntatori atti ad
assicurare che l'array possa essere usato con i regolari operatori [].
L'allocazione della memoria avviene per mezzo di calloc().
E' responsabilita' del chiamante liberare con free() la memoria allocata.
In caso di fallimento restituisce NULL (l'unica ragione di fallimento possibile
e' l'impossibilita' da parte di calloc() d'allocare la memoria necessaria).

PARAMETRI
  dim_elem  dimensioni, in byte, di ciascuno degli elementi dell'array;
            e' comodo usare sizeof() per sapere quale valore passare in questo
            primo parametro

  w         quantita' delle celle sulla dimensione X dell'array (width)
  h         quantita' delle celle sulla dimensione Y dell'array (height)
  d         quantita' delle celle sulla dimensione Z dell'array (depth)

VALORE DI RITORNO
  void*     puntatore che punta al blocco di memoria allocato;
            NULL in caso d'errore

ESEMPIO

int main() {
    // le coordinate sono passate come...    d  w  h
    int ***a3d = crea_array_2d( 4, 2, 3, sizeof(int) );

    if( a3d ) {
        // a3d punta ora a una array 3D di 4x2x3 celle

        //  z  y  x <= ma si usano in ordine inverso!
        a3d[2][0][1] = -27;
        a3d[1][1][2] =  15;

        //        z  y  x              z  y  x
        printf( "[2][0][1]: %d\n", a3d[2][0][1];
        printf( "[1][1][2]: %d\n", a3d[1][1][2];

        free( a3d );
        a3d = NULL;
    }

        free( a3d );
        a3d = NULL;
    }

    return 0;
}
==============================================================================*/

void *T1_crea_array_3d( size_t w, size_t h, size_t d, size_t dim_el );


/*==============================================================================
Divide l'area delimitata dal parametro limiti in qr righe per qc colonne, quindi
crea qr*qc RECT e li imposta in modo che definiscano una griglia regolare entro
il RECT *limiti.
Restituisce il puntatore al punto di accesso della matrice, ovvero allo spazio
di memoria allocato con calloc() nel quale la matrice e' contenuta.
Spetta al chiamante deallocare con free() la memoria puntata dal valore di
ritorno.
==============================================================================*/

RECT **T1_CreaMatriceRECT( const RECT *limiti, size_t qr, size_t qc );


/*==============================================================================
Crea in un'area di memoria dinamica allocata con malloc() una versione WCHAR
della stringa C di char cStr.
Se la funzione va a buon fine, restituisce il puntatore alla memoria allocata,
che contiene la versione WCHAR di cStr, altrimenti restituisce NULL. Spetta al
chiamante deallocare con free() la memoria puntata dal valore di ritorno.
La funzione fallisce restituendo NULL solo cStr e' NULL o se malloc() fallisce a
sua volta.
==============================================================================*/

WCHAR *T1_CStr2WStr( LPCSTR cStr );


/*==============================================================================
Analizza la stringa str in cerca delle ricorrenze di caratteri c multipli
adiacenti. Conserva uno solo dei caratteri c per ogni gruppo di caratteri
adiacenti.
==============================================================================*/

void T1_elimina_caratteri_multipli_adiacenti( char *str, char c );


/*==============================================================================
Analizza la stringa str in cerca delle ricorrenze della stringa daEliminare e,
ovviamente le elimina una ad una.
==============================================================================*/

void T1_elimina_occorrenze_stringa( char *str, const char *daEliminare );


/*==============================================================================
Elimina "sul posto" i caratteri spaziatori consecutivamente multipli, facendo
inoltre in modo che la stringa modificata non contenga mai un carattere
spaziatore nella sua prima ne' nella sua ultima posizione.
Restituisce la dimensione della stringa modificata.
In caso d'errore, restituisce il valore massimo rappresentabile per mezzo del
tipo size_t.
==============================================================================*/

size_t T1_elimina_spaziature_multiple_adiacenti( char *str, char spaziatore );


/*==============================================================================
Analizza la stringa str in cerca di coppie consecutive dei delimitatori indicati
da delim_1 e delim_2, eliminando le porzioni di str contenute tra i delimitatori
rilevati. I delimitatori vengono a loro volta eliminati o preservati secondo
quanto indicato tramite i parametri preserva_delim_1 e preserva_delim_2.
==============================================================================*/

void T1_elimina_tra_stringhe( char *str,
                              const char *delim_1, const char *delim_2,
                              int preserva_delim_1, int preserva_delim_2 );


/*==============================================================================
Con la funzione T1_trova_elemento() individua l'elemento richiesto nell'ambito
della stringa C elementi (che si suppone contenga una lista di sottostringhe
separate dal carattere indicato tramite il parametro separatore), quindi ne
effettua una copia in uno spazio di memoria dinamica appositamente allocato con
malloc() eliminando dalla stringa C elementi quanto appena copiato.
Se la funzione ha successo restituisce il puntatore allo spazio di memoria
dinamica nel quale e' stato copiato l'elemento richiesto (spetta al chiamante
liberare la memoria allocata con free()).
Se la funzione fallisce restituisce NULL.
==============================================================================*/

char *T1_estrai_elemento( char *elementi, char separatore, int indice );


/*==============================================================================
Converts a text file containing a list of substrings separated from each other
by single-character separators into an array of C strings contained in a single
block of memory.

Parameters

  size_t *itemsQuantity     (output)
                            The function will put the amount of items in the
                            resulting array into *itemsQuantity. If this
                            parameter is NULL the function fails.

  const char *fileName      (input)
                            The name of the text file to load as an array of
                            strings.
                            The file must contain a list of substrings separated
                            from each other by single-character separators.
                            If this parameter is NULL or points to an empty
                            string the function fails.

  char itemsSeparator       (input)
                            The single-character separator used to recognize
                            the substrings in itemsList.

Return Value

  If the function succeeds returns a char** pointing to the array of C strings
  got from the file fileName. As the array "lives" in dynamic memory, the caller
  MUST get rid of this pointer via the standard function free() when he see fit.
  If the function failes returns NULL. The function fails if itemsQuantity is
  NULL, or the file fileName is not accessible, or dynamic memory allocation
  isn't possible.

Note
  In order to use this function your program must include stdio.h, stdlib.h and
  string.h.
==============================================================================*/

const char **T1_file2array( size_t *itemsQuantity,
                            const char *fileName, char itemsSeparator );


/*==============================================================================
Il parametro tipoFile identifica il tipo di file che sar riconosciuto dalla
finestra di navigazione e lestensione del quale sar usata come estensione di
default, se del caso.
Il tipo specificato deve avere la forma (ad esempio) "*.txt".
Per indicare tipi multipli, si usi la forma (ad esempio) "*.txt;*.rtf".
Se questo parametro  NULL saranno accettati file di ogni tipo e non sar
attribuita alcuna estensione di default.

Il parametro nomeFileDflt identifica il nome del file usato come default nella
finestra del tipo Salva. Se nomeFileDflt  un puntatore valido, la finestra
usata sar una finestra del tipo Salva; se  NULL, la finestra usata sar una
finestra del tipo Apri.

In caso di successo, in *nomeFile viene collocato il puntatore a uno spazio di
memoria allocata dinamicamente con calloc() che contiene il nome del file
rilevato. In caso di insuccesso o di annullamento delloperazione, nessuna
memoria  stata allocata e *nomeFile non viene modificato.

La funzione restituisce uno tra i seguenti tre valori:
    0  operazione riuscita
   -1  si  verificato un errore
    1  operazione annullata

N.B. E' necessario collegare la libreria di Windows comdlg32 e includere il file
     di intestazione commdlg.h
==============================================================================*/

int T1_IdentificaFile ( HWND hwnd, const char *tipoFile,
                        const char *nomeFileDflt, char **nomeFile );


/*==============================================================================
Immette negli appunti riferibili alla finestra hFin la stringa passata tramite
il parametro str.
E' possibile passare in lStr la quantita' dei caratteri da immettere negli
appunti. Se lStr non e' ((UINT)-1) la stringa puntata da str puo' anche non
essere zero terminated. Se lStr e' ((UINT)-1) la funzione ricava autonomamente
la lunghezza della stringa C (zero terminated) passata in str.
Se loperazione va a buon fine, ImpostaAppuntiTxt() restituisce TRUE, altrimenti
restituisce FALSE.
==============================================================================*/

BOOL T1_ImpostaAppuntiTxt( HWND hFin, const char *str, UINT lStr );


/*==============================================================================
Riceve, tramite il parametro s, un puntatore ad una stringa "zero terminated" e
la "rovescia" in loco invertendone l'ordine dei caratteri.
Restituisce lo stesso puntatore ricevuto tramite il parametro s.
==============================================================================*/

char *T1_inverti_stringa( char *s );


/*==============================================================================
Converts a C string formatted as a list of single-character-separated substrings
into an array of C strings contained in a single block of memory.

Parameters

  size_t *itemsQuantity     (output)
                            The function will put the amount of items in the
                            resulting array into *itemsQuantity. If this
                            parameter is NULL the function fails.

  const char *itemsList     (input)
                            This parameter points to the C string formatted as
                            a list of single-character-separated substrings to
                            convert into an array of C strings contained in a
                            single block of memory. If this parameter is NULL
                            the function fails.

  char itemsSeparator       (input)
                            The single-character separator used to recognize
                            the substrings in itemsList.

Return Value

  If the function succeeds returns a char** pointing to the array of C strings
  got from itemsList. As the array "lives" in dynamic memory, the caller MUST
  get rid of this pointer via the standard function free() when he sees fit.
  If the function failes returns NULL. The function fails if itemsQuantity is
  NULL, or itemsList is NULL, or dynamic memory allocation isn't possible.

Note:
  In order to use this function your program must include stdlib.h and string.h.
==============================================================================*/

const char **T1_items2array( size_t *itemsQuantity,
                             const char *itemsList,
                             char itemsSeparator );


/*==============================================================================
Legge una riga d'un file aperto in lettura testuale a partire dalla posizione
corrente nello stream correlato. La fine della riga viene riconosciuta per la
presenza del carattere '\n' o per il raggiungimento della fine del file.
Alloca dinamicamente lo spazio strettamente necessario per caricare la stringa,
legge la stringa nello spazio allocato e colloca in *riga il puntatore allo
spazio allocato. Spetta al chiamante liberare con free() la memoria allocata.
Restituisce la lunghezza della stringa letta, escluso il carattere terminatore.

CONDIZIONI D'ERRORE
Se si verifica un errore, in ogni caso il segnaposizione del file rimane nella
sua posizione originaria e la funzione restituisce (size_t)-1. Inoltre, se...

  1. il parametro riga e' NULL
        la funzione non alloca alcuna memoria;
        *riga non viene modificato (ovviamente: il suo puntatore e' NULL!)

  2. il parametro file e' NULL
        la funzione non alloca alcuna memoria;
        *riga e' impostato su NULL

  3. non e' possibile allocare memoria
        la funzione non alloca alcuna memoria (ma va'?);
        *riga e' impostato su NULL

  4. il file non contiene dati oltre la posizione corrente
        la funzione non alloca alcuna memoria (ma va'?);
        *riga e' impostato su NULL

  5. non e' possibile rilevare la posizione corrente nello stream
        la funzione non alloca alcuna memoria (ma va'?);
        *riga e' impostato su NULL

==============================================================================*/

size_t T1_leggi_riga_file( FILE *file, char **riga );


/*==============================================================================
Imposta la localizzazione italiana per le applicazioni tramite console del C.
==============================================================================*/

void T1_localizzazione_italiana( void );


/*==============================================================================
Apre il file nome_file con requisiti di accesso "r", quindi ne analizza il
contenuto in cerca della quantita' dei caratteri contenuti nella riga piu'
lunga. Chiude il file senza modificarlo e restituisce il valore rilevato.
==============================================================================*/

size_t T1_lunghezza_massima_riga_file( const char *nome_file );


/*==============================================================================
Riceve gli indirizzi di tre RECT.

    rEst    un RECT di riferimento entro il quale sara' centrato, scalandolo in
            modo che sia il piu' grande possibile, l'altro RECT puntato da
            rInterno

    rInt    il rect da centrare, scalato in modo che sia il piu' grande
            possibile, entro l'altro RECT puntato da rEsterno; l'indirizzo puo'
            coincidere con quello di rMassimizzato

    rMax    l'indirizzo a un RECT che ricevera' la versione centrata e scalata
            del RECT puntato da rInterno; questo indirizzo puo' coincidere con
            quello di rInterno, nel qual caso il RECT puntato da rInterno sara'
            modificato anche se  rInterno e' un parametro const

Restituisce un valore di tipo BOOL che e' TRUE se la funzione e' andata a buon
fine, FALSE se si sono verificati errori. L'unico errore che puo' verificarsi
deriva dal passaggio di uno o piu' puntatori NULL.
==============================================================================*/

BOOL T1_MassimizzaRectInRect( const RECT *rEst, const RECT *rInt, RECT *rMax );


/*==============================================================================
Mostra nella console del C un menu' costituito da un titolo (opzionale) seguito
da una riga separatrice e da una serie di altre voci numerate. Si occupa quindi
di assicurare la restituzione di un valore coerente con le caratteristiche del
menu', insistendo nella richiesta di una nuova scelta finche' non si sceglie una
voce di menu' esistente. Ad esempio...

 int scelta = T1_menu_va("MENU DI PROVA",4,"Uscita","Voce 1","Voce 2","Voce 3");

La chiamata appena riportata da' come risultato questo menu':

    MENU DI PROVA               |
  =================             |
    1. Voce 1                   | Un menu' di questo tipo, restituisce sempre
    2. Voce 2                   | un valore compreso tra 0 e 3, a meno che siano
    3. Voce 3                   | stati mal formattati i parametri.
    0. Uscita                   |
  Scegli una voce dal menu:     |

==========

PARAMETRI

const char *titolo  Una stringa C che viene usata come titolo del menu'.
                    Dopo il titolo, viene tracciata una riga separatrice
                    costituita da una serie di caratteri '='.
                    Se questo parametro e' NULL, il menu' viene presentato senza
                    un titolo, e la riga separatrice non viene tracciata.

int qVoci           Un valore intero >=1 che indica la quantita' delle voci
                    complessivamente presenti nel menu' (inclusa quella di
                    uscita).

const char* ...     Una serie di puntatori a stringhe C, in quantita' pari al
                    valore indicato in qVoci.
                    La prima stringa viene usata come etichetta per la voce di
                    uscita dal menu', e collocata in coda al menu' stesso,
                    numerandola con 0.
                    Ciascuna delle stringhe successive viene usata come
                    etichetta delle altre voci, numerandole a partire da 1,
                    secondo il loro ordine di successione tra i parametri.
                    Nessuno dei puntatori puo' essere NULL.

VALORE DI RITORNO

int                 Un valore numerico intero che e' 0 se e' stata scelta la
                    voce riservata all'uscita, maggiore di 0 se e' stata scelta
                    un'altra voce, sempre compresa entro quelle definite tramite
                    il secondo parametro e quelli successivi.
                    Il valore di errore -1 viene restituito solo se il secondo
                    parametro e' <1 o anche uno solo tra i parametri successivi
                    e' NULL.
==============================================================================*/

int T1_menu_va( const char *titolo, int qVoci, ... );


/* =============================================================================
Verifica che nella stringa s ad ogni parentesi aperta pa corrisponda una
parentesi chiusa pc, e che non si ripresentino parentesi aperte prima che la
parentesi aperta precedente sia stata chiusa.
Se le parentesi sono bilanciate restituisce 1, in caso contrario restituisce 0.

Esempi:

    |    "<prova>"           -> 1    |    "<<prova><prova>>"  -> 0    |
    |    "<prova><prova>"    -> 1    |    "<<prova>"          -> 0    |
    |    "<><><>"            -> 1    |    "<prova>>"          -> 0    |
    |    "prova"             -> 1    |    "<<prova>>"         -> 0    |
    |    ""                  -> 1    |                                |
==============================================================================*/

int T1_parentesi_bilanciate( const char *s, char pa, char pc );


/*==============================================================================
Estrae a sorte un numero pseudocasuale compreso tra 0x00000000 e 0xFFFFFFFF.
==============================================================================*/

uint32_t T1_rand_32( void );


/*==============================================================================
Estrae a sorte un numero pseudocasuale compreso tra 0.0 e 1.0.
==============================================================================*/

double T1_rand_double( void );


/*==============================================================================
Permette di rilevare le dimensioni di un'immagine BITMAP. Se tutto funziona a
dovere, restituisce TRUE, altrimenti restituisce FALSE. Se le cose vanno a buon
fine, le dimensioni della bitmap sono immagazzinate in *dim; se le cose vanno
storte *dim rimane invariato.
==============================================================================*/

BOOL T1_RilevaDimensioniBitmap( HBITMAP hBmp, SIZE *dim );


/*==============================================================================
Converte un colore espresso come rosso (r), blu (b) e e giallo (y) con valori
compresi tra 0.0 e 1.0 per ciascuna componente nel corrispondente colore
espresso come COLORREF (0x00BBGGRR).
==============================================================================*/

COLORREF T1_rby2rgb( double r, double b, double y );


/*==============================================================================
Dati un colore di primo piano e uno di sfondo, provvede a miscelarli in modo che
il valore di alpha venga "incorporato" negli altri canali.
Il colore derivante dalla miscela viene restituito come valore di ritorno.
Il colore restituito come valore di ritorno ha sempre il canale alpha impostato
su 0xFF (totalmente opaco).

NOTA: da' per scontato che lo sfondo sia un colore completamente opaco
      (l'eventuale trasparenza non viene neppure presa in considerazione).
==============================================================================*/

ARGB T1_RemoveAlpha( ARGB colore, ARGB sfondo );


/*==============================================================================
Carica in memoria i dati in formato TEXT contenuti negli appunti riferibili alla
finestra hFin (hFin puo' essere NULL se gli appunti non riguardano una
particolare finestra).
Il valore restituito e' il puntatore a uno spazio di memoria allocato tramite
malloc() nel quale e' contenuta la stringa C ricavata dagli appunti. Se non
esistono appunti leggibili o se e' impossibile allocare spazio in memoria, la
funzione restituisce NULL.
In uscita, inoltre, se il parametro lTesto non e' NULL *lTesto riporta la
quantita' di caratteri contenuta nella stringa restituita come valore di
ritorno.
==============================================================================*/

char *T1_RilevaAppuntiTxt( HWND hFin, UINT *lTesto );


/*==============================================================================
L'area utile dello schermo e' quella che nella documentazione di Win32 viene
denominata "work area" e consiste nella porzione di schermo potenzialmente a
disposizione di un programma. Ad esempio, l'area utile puo' essere piu' o meno
estesa a seconda della condizione di visibilita' della barra del menu Start di
Windows.
Questa funzione riporta all'indirizzo segnalato tramite il parametro
corrispondente i dati rilevati. Alcuni parametri possono essere NULL, a
condizione che non lo siano tutti. Se un parametro e' NULL, il dato
corrispondente non viene segnalato al chiamante.
In  caso di successo viene restituito TRUE, mentre in caso di fallimento viene
restituito FALSE.
La funzione fallisce e restituisce FALSE in due situazioni:
  - se TUTTI i parametri sono NULL;
  - se la funzione di Win32 SystemParametersInfo() fallisce a sua volta.
==============================================================================*/

BOOL T1_RilevaAreaUtileSchermo( LPRECT area_utile, LPLONG wMax, LPLONG hMax );


/*==============================================================================
Confrontando il window RECT di una finestra e il suo client RECT, ricava lo
"spessore" dei bordi, collocandoli nei quattro campi del RECT *bordi.
In  caso di successo viene restituito TRUE, mentre in caso di fallimento viene
restituito FALSE.
La funzione fallisce e restituisce FALSE in cinque situazioni:
  - se hwnd non identifica una finestra valida di Win32;
  - se bordi e' NULL;
  - se la funzione di Win32 GetWindowRect() fallisce;
  - se la funzione di Win32 GetClientRect() fallisce;
  - se la funzione di Win32 ClientToScreen() fallisce.
==============================================================================*/

BOOL T1_RilevaBordiFinestra( HWND hwnd, LPRECT bordi );


/*==============================================================================
Rileva le dimensioni del carattere c per il font correntemente selezionato nel
device context hdc e le inserisce in *dim.
Se la funzione va a buon fine, restituisce TRUE, altrimenti restituisce FALSE.
La funzione fallisce e restituisce FALSE in due situazioni:
  - se dim e' NULL
  - se la funzione di Win32 GetTextExtentPoint32() fallisce a sua volta.
==============================================================================*/

BOOL T1_RilevaDimensioniCarattere( HDC hdc, CHAR c, LPSIZE dim );


/*==============================================================================
Rileva le dimensioni della stringa t per il font correntemente selezionato nel
device context hdc e le inserisce in *dim.
Se la funzione va a buon fine, restituisce TRUE, altrimenti restituisce FALSE.
La funzione fallisce e restituisce FALSE in due situazioni:
  - se dim e' NULL o t e' NULL
  - se la funzione di Win32 GetTextExtentPoint32() fallisce a sua volta.
==============================================================================*/

BOOL T1_RilevaDimensioniTesto( HDC hdc, LPCSTR t, LPSIZE dim );


/*==============================================================================
Entro i limiti imposti dal parametro r, traccia nel Device Context rappresentato
dal parametro hdc un riquadro bordato che puo' essere piano, incassato o in
rilievo.

Il colore dello sfondo del riquadro e' dato ovviamente dal parametro colore.

Lo spessore del bordo del riquadro e' dato ovviamente dal parametro
spessore_bordo. Lo spessore del bordo viene inteso come valore assoluto, per cui
un valore di spessore_bordo negativo viene reso comunque positivo internamente
alla funzione. Se spessore bordo e' 0 (zero), il bordo non viene tracciato.

Il parametro direzione_rilievo determina la "sporgenza" del bordo:
  - se e' positivo il riquadro e' in rilievo
  - se e' negativo il riquadro e' incassato
  - se e' 0 (zero), il riquadro ha un bordo piano

Con direzione_rilievo uguale a zero, il bordo e' scurito se spessore_bordo e'
negativo e schiarito se spessore_bordo e' positivo.

Il livello di schiarimento/scurimento dei bordi e' determinato dal parametro
entita_rilievo. Il valore di entita_rilievo puo' variare da 0x00 a 0xFF.
==============================================================================*/

BOOL T1_RiquadroBordato( HDC hdc, const RECT *area,
                         COLORREF colore, INT16 spessore_bordo,
                         CHAR direzione_rilievo, BYTE entita_rilievo);


/*==============================================================================
Scambia i nomi di due file identificati tramite i due parametri f1 e f2, senza
modificarne in alcun modo il contenuto.
Se lo scambio dei nomi va a buon fine restituisce 1; in caso di errore, 0.
==============================================================================*/

int T1_scambia_nomi_file( const char *f1, const char *f2 );


/*==============================================================================
Applica uno schiarimento di entita' definita dal parametro variazione al colore
indicato dal parametro colore. Restituisce il colore risultante come valore di
ritorno.

Se il parametro schiarisci e' TRUE, il colore viene schiarito. Il cambiamento
avviene come se il colore, con canale alfa impostato su (0xFF-variazione),
venisse sovrapposto a uno sfondo bianco.

Se il parametro schiarisci e' FALSE, il colore viene scurito. Il cambiamento
avviene come se il colore, con canale alfa impostato su (0xFF-variazione),
venisse sovrapposto a uno sfondo nero.
==============================================================================*/

COLORREF T1_SchiarisciColore( COLORREF colore,BYTE variazione,BOOL schiarisci );


/*==============================================================================
Riceve un colore espresso come ARGB e ne restituisce in le componenti.
==============================================================================*/

void T1_ScomponiARGB( ARGB colore, BYTE *a, BYTE *r, BYTE *g, BYTE *b );


/*==============================================================================
Riceve un colore espresso come COLORREF e ne restituisce le componenti.
Da notare che in un COLORREF la componente a (alpha) non e' significativa.
==============================================================================*/

void T1_ScomponiCOLORREF( COLORREF colore, BYTE *a, BYTE *r, BYTE *g, BYTE *b );


/*==============================================================================
Inserisce in *cPt le coordinate locali corrispondenti alle coordinate a schermo
*sPt. I puntatori sPt e cPt possono puntare alla stessa struttura POINT.
In  caso di successo viene restituito TRUE, mentre in caso di fallimento viene
restituito FALSE.
La funzione fallisce e restituisce FALSE in tre situazioni:
  - se anche uno solo dei parametri e' NULL;
  - se hwnd non identifica una finestra valida di Win32;
  - se la funzione di Win32 ScreenToClient() fallisce a sua volta.
==============================================================================*/

BOOL T1_ScreenPoint2ClientPoint( HWND hwnd, CONST LPPOINT sPt, LPPOINT cPt );


/*==============================================================================
Inserisce in *sRect la versione riferita allo schermo del RECT locale *cRect.
I puntatori cRect e sRect possono puntare alla stessa struttura RECT.
In  caso di successo viene restituito TRUE, mentre in caso di fallimento viene
restituito FALSE.
La funzione fallisce e restituisce FALSE in tre situazioni:
  - se anche uno solo dei parametri e' NULL;
  - se hwnd non identifica una finestra valida di Win32;
  - se la funzione di Win32 ScreenToClient() fallisce a sua volta.
==============================================================================*/

BOOL T1_ScreenRect2ClientRect( HWND hwnd, LPCRECT sRect, LPRECT cRect );


/*==============================================================================
Riceve in str una stringa c che si suppone rappresenti una frazione e ne
effettua internamente una copia nella quale sostituisce tutti i caratteri ','
con caratteri '.' (la memoria allocata viene comunque liberata prima che la
funzione ritorni).
Applica strtod() alla copia di str per ricavarne il numeratore.
Salta gli eventuali spazi presenti dopo il numeratore.
Verifica che sia presente l'operatore '/'.
Applica strtod() a cio' che segue l'operatore '/' per ricavarne il denominatore.
Verifica che non esistano caratteri "spuri" dopo il denominatore,
Se numeratore e denominatore sono stati "estratti" correttamente, li colloca
ripettivamente in *n e *d.

In caso di successo delle operazioni restituisce 1 (come per "true").
In caso di insuccesso restituisce 0 (come per "false").

La funzione fallisce se:
  - str e' NULL o n e' NULL o d e' NULL
  - risulta impossibile effettuare la copia temporanea di str
  - strtod() fallisce nel rilevare il numeratore o il denominatore
  - l'operatore '/' e' assente o non e' collocato correttamente
  - str contiene caratteri "spuri" estranei alla frazione da convertire
==============================================================================*/

int T1_str_in_elementi_frazione( const char *str, double *n, double *d );


/*==============================================================================
Colloca in buff (che deve poter contenere una stringa C di ALMENO qBit caratteri
piu' uno) una stringa costituita da '0' e '1' che corrisponde al valore di n
espresso in forma binaria.

NOTA: Il puntatore restituito e' lo stesso passato come parametro buff.
      Se qBit non e' compreso tra 1 e 64 nel buffer viene collocata una stringa
      C vuota ("").
==============================================================================*/

char *T1_str_binario( char *buff, uint64_t n, int qBit );


/*==============================================================================
Una versione di strcat() che agisce su una stringa allocata dinamicamente.

PARAMETRI

   sd   stringa dinamica
        Indirizzo del puntatore a uno spazio di memoria allocato dinamicamente
        con malloc(), calloc() o realloc() nel quale risiede la stringa in coda
        alla quale effettuare il concatenamento. Puo' essere NULL.

   lSd  lunghezza della stringa dinamica *sd
        Indirizzo di un valore size_t nel quale puo' essere collocata la
        quantita' di caratteri dopo la quale concatenare la stringa da
        aggiungere sa. Il puntatore lSd puo' essere NULL, cosi' come il valore
        puntato puo' essere ((size_t)-1) - in entrambi i casi, la funzione
        tenta di ricavare la lunghezza della stringa sd tramite strlen(), ma se
        *sd e' NULL, presuppone per *sd una lunghezza 0.
        Se il puntatore non e' NULL, in uscita *lSd contiene la dimensione dello
        spazio di memoria nel quale risiede la stringa *sd.

   sa   stringa da aggiungere
        Puntatore alla stringa da concatenare a *sd. Puo' essere NULL.

   lSa  lunghezza della stringa da aggiungere
        Se e' ((size_t)-1) e sa non e' NULL, la funzione impiega strlen() per
        ricavare autonomamente la lunghezza della stringa da concatenare.

==============================================================================*/

int T1_strcat_dinamico( char **sd, size_t *lSd, const char *sa, size_t lSa );


/*==============================================================================
Questa funzione si occupa di creare in memoria dinamica un duplicato della
stringa str. In uscita, str risulta invariata.

    str     str: stringa; un puntatore a const char che punta ad una stringa
            C valida (zero terminated); dopo l'elaborazione, il contenuto
            della stringa rimane invariato

    lStr    lStr: lunghezza della stringa; se il valore di questo parametro
            e' ((size_t)-1) la funzione usa strlen() per ricavare la
            lunghezza di str, in caso contrario usa il valore passato per
            allocare il buffer nel quale collocare la copia di str

Valore di ritorno: il puntatore allo spazio di memoria allocato con malloc()
                   nel quale e' stato collocato il duplicato della stringa
                   passata in str. Spetta al chiamante liberare con free() la
                   memoria puntata.
==============================================================================*/

char *T1_strdup_dinamico( const char *str, size_t lStr );


/*==============================================================================
Dato un valore intero positivo, lo converte in una stringa C che lo rappresenta
impiegando le cifre contenute nell'array puntato dal secondo parametro.

La base di numerazione utilizzata per la conversione corrisponde alla lunghezza
dell'array cifre. Le cifre possono essere costituite da caratteri qualsiasi.

NOTA: ogni cifra DEVE essere univoca (case sensitive) ma la funzione non
      effettua alcun controllo relativamente a questa condizione, dando per
      scontato che la validita' delle cifre passate sia gia' stata accertata
      dal chiamante

PARAMETRI

  uint32_t valore       input
                        il valore numerico intero positivo da convertire

  const char *cifre     input
                        puntatore a un array di base caratteri che
                        rappresentano nell'ordine, l'intero insieme delle cifre
                        impiegate nel sistema di numerazione

  uint8_t base          input
                        la quantita' dei caratteri contenuti nell'array cifre;
                        il valore minimo ammesso e' 2

  const char *n_str     output
                        la stringa C che contiene la rappresentazione del
                        valore intero positivo

VALORE DI RITORNO

Se la funzione assolve il suo compito restituisce la quantita' delle cifre
inserite nella stringa in uscita (valore, quindi, positivo da interpretare come
"true"), altrimenti restituisce 0 (da interpretare come "false").

La funzione puo' fallire se...
  - uno qualsiasi dei parametri e' NULL
  - n_str o cifre sono stringhe vuote
  - base e' minore di 2
==============================================================================*/

int T1_stringa_da_valore( uint32_t valore,
                          const char *cifre, uint8_t base,
                          char *n_str );


/*==============================================================================
PRESUPPOSTI

    Il parametro s punti a una stringa C valida.
    Il parametro ll sia compreso entro i limiti della stringa s o sia -1.
    (se ll e' -1, viene usata l'intera stringa; se ll eccede i limiti della
    stringa, viene comunque usata solo la stringa, senza superarne i limiti).
    I parametri d1 e d2 siano due puntatori validi.

COMPITO

    Individuare, nell'ambito della stringa s, gli indici del primo carattere
    delimitatore e del carattere delimitatore immediatamente successivo.

ESITO

    Se entrambi i caratteri vengono individuati, la ricerca ha avuto successo.
    Se non viene individuato alcun carattere o se ne viene individuato uno solo,
    la ricerca e' fallita.
    Se s e' NULL, la ricerca e' impossibile e quindi si considera fallita.
    Se d1, d2 o entrambi sono NULL, la comunicazione degli esiti della ricerca
    e' impossibile, e dunque la ricerca stessa si considera fallita.

VALORE DI RITORNO

    Se la ricerca ha successo, viene restituito 1.
    Se la ricerca fallisce, viene restituito 0.

PARAMETRI

input:

    s       una stringa C
    d       il carattere delimitatore del quale trovare una coppia
    ll      la lunghezza oltre la quale non spingersi nell'analisi di s

output:

    d1      l'indice del primo delimitatore a partire da s
    d2      l'indice del secondo delimitatore a partire da s

==============================================================================*/

int T1_trova_coppia_delimitatori( const char *s, size_t ll, char d, /* input  */
                                  size_t *d1, size_t  *d2 );        /* output */


/*==============================================================================
Analizza la stringa C elementi, che si suppone essere una lista di sottostringhe
separate dal carattere indicato tramite il parametro separatore, in cerca dello
elemento indicato dal parametro indice (zero based).
Se l'elemento richiesto esiste (ovvero se indice e' minore della quantita' degli
elementi presenti nella lista) la funzione ne restituisce il puntatore e colloca
in *lunghezza_elemento la quantita' dei caratteri che costituiscono la
sottostringa dell'elemento trovato (NOTA: lunghezza_elemento puo' essere NULL,
nel qual caso la quantita' dei caratteri non viene notificata al chiamante).
Se l'elemento richiesto non esiste la funzione restituisce NULL e
*lunghezza_elemento non viene modificato.
==============================================================================*/

const char *T1_trova_elemento( const char *elementi,
                               char separatore, size_t indice,
                               size_t *lunghezza_elemento );


/*==============================================================================
Usando le impostazioni correnti del device context hdc, traccia il testo secondo
quanto definito nella struttura aTxt (la struttura T1_AreaTesto_struct e'
definita in testa a questo file).
Se hdc e' NULL o aTxt e' NULL, la funzione non fa nulla.
==============================================================================*/

void T1_TracciaAreaTesto( HDC hdc, const T1_AreaTesto_struct *aTxt );


/*==============================================================================
Data una stringa che contiene la rappresentazione di un valore intero positivo
ottenuta con le cifre contenute nell'array puntato dal secondo parametro, la
converte nell'uint32_t corrispondente.

La base di numerazione utilizzata per la conversione corrisponde alla lunghezza
dell'array cifre. Le cifre possono essere costituite da caratteri qualsiasi.

NOTA: ogni cifra DEVE essere univoca (case sensitive) ma la funzione non
      effettua alcun controllo relativamente a questa condizione, dando per
      scontato che la validita' delle cifre passate sia gia' stata accertata
      dal chiamante

PARAMETRI

  const char *n_str     input
                        la stringa C che contiene la rappresentazione del valore
                        intero positivo

  const char *cifre     input
                        puntatore a un array di base caratteri che
                        rappresentano nell'ordine, l'intero insieme delle cifre
                        impiegate nel sistema di numerazione

  uint8_t base          input
                        la quantita' dei caratteri contenuti nell'array cifre;
                        il valore minimo ammesso e' 2

  uint32_t *valore      output
                        il valore numerico intero positivo ottenuto dalla
                        conversione viene collocato all'indirizzo puntato da v

VALORE DI RITORNO

Se la funzione assolve il suo compito restituisce 1 (da interpretare come
"true"), altrimenti restituisce 0 (da interpretare come "false").

La funzione puo' fallire se...
  - uno qualsiasi dei parametri e' NULL
  - n_str o cifre sono stringhe vuote
  - in n_str sono presenti caratteri che mancano in cifre
  - base e' minore di 2
==============================================================================*/

int T1_valore_da_stringa( const char *n_str,
                          const char *cifre, uint8_t base,
                          uint32_t *valore );


#ifdef __cplusplus
}
#endif

#endif // LIBRERIA_TUTTO_IN_UNO_H_INCLUDED
