#include "scrittura_sintesi.h"


static char *abbrevia_colore( char *s ) {
    if( s && '#'==*s && 7==strlen(s) ) {
        if( *(s+1)==*(s+2) && *(s+3)==*(s+4) && *(s+5)==*(s+6) )
            { *(s+2)=*(s+3);  *(s+3)=*(s+5); *(s+4)='\0'; }
    }

    return s;
}


static char *abbrevia_double( char *s ) {
    if( s ) {
        int l = strlen( s );

        if( 0!=l ) {
            if( strchr(s,'.') )
                while( '0'==s[l-1] ) s[--l]='\0'; /* elimina gli zeri finali */
            if( '.'==s[l-1] )
                s[--l]='\0'; /* elimina l'eventuale virgola rimasta in coda */
            if( '\0'==s[0] )
                { s[0] = '0'; s[1] = '\0'; } /* stringa vuota? zero! */
        }
    }

    return s;
}


static size_t conta_tags_classe_corrente( TagSvg *t, size_t qt ) {
/* conta quanti sono i tag nell'array t che condividono t[0].classe */
    size_t qEl = 0;
    while( t[qEl].classe==t[0].classe ) qEl++;
    return qEl;
}


static int fattibilita_def( int classe ) {
    switch( classe ) {
        case kAccidental:
        case kArpeggio:
        case kArticulation:
        case kBracket:
        case kClef:
        case kDynamic:
        case kHook:
        case kKeySig:
        case kNote:
        case kNoteDot:
        case kRest:
        case kTimeSig:
            return 1;
        default:
            return 0;
    }
}


static void assegna_id_path( TagSvg *t, size_t qt ) {
    int    id = 0;
    size_t  i = 0;
    size_t  j = 0;

    /* marca alcuni tag come da etichettare */

    while( i<qt ) {
        if( fattibilita_def(t[i].classe) )
            t[i].id = -2; /* da etichettare */
        else t[i].id = -1; /* da NON etichettare */
        ++i;
    }

    i = 0; /* torna all'inizio dell'array t */

    while( i<qt ) {
        if( -2==t[i].id ) {
            t[i].id   = id;
            t[i].tipo = kUse;

            j = i+1;

            while( j<qt ) {
                if( -2==t[j].id && 0==strcmp(t[i]._d,t[j]._d) ) {
                    t[j].id   = id;
                    t[j].tipo = kUse;
                }

                ++j;
            }

            ++id;
        }

        ++i;
    }
}


static size_t scrivi_defs( char *s, TagSvg *t, size_t qt ) {
    char  *p  = NULL;
    int    id = 0;
    size_t  i;

    for( p=s, i=0; i<qt; ) {
        if( t[i].id==id ) { /* occorre creare il def */
            p += sprintf( p, "<path id=\"%d\" d=\"", id );
            p += sprintf( p, "%s\"/>\n", t[i]._d );
            while( t[i].id==id ) ++i;
            ++id;
            continue;
        }

        ++i;
    }

    return p-s;
}


static int confronta_uint32( const void *p1, const void *p2 ) {
    uint32_t u1 = *((uint32_t*)p1);
    uint32_t u2 = *((uint32_t*)p2);
    return u1>u2 ? 1 : (u1<u2?-1:0);
}


static uint32_t rileva_uint32_piu_frequente( uint32_t *vv, size_t qv ) {
    uint32_t v=*vv, vc=*vv;
    size_t i=0, f=0, fMax=0;

    while( i<qv ) {
        while( i<qv&&vv[i]==vc ) { ++i; ++f; }

        if( f>fMax ) { fMax=f; v=vv[i-1]; }

        vc = vv[i];
        f = 0;
    }

    return v;
}

/*
static int confronta_int( const void *p1, const void *p2 ) {
    int i1 = *((int*)p1);
    int i2 = *((int*)p2);
    return i1>i2 ? 1 : (i1<i2?-1:0);
}


static int rileva_int_piu_frequente( int *vv, size_t qv ) {
    int v=*vv, vc=*vv;
    size_t i=0, f=0, fMax=0;

    while( i<qv ) {
        while( i<qv&&vv[i]==vc ) { ++i; ++f; }

        if( f>fMax ) { fMax=f; v=vv[i-1]; }

        vc = vv[i];
        f = 0;
    }

    return v;
}
*/

static int confronta_double( const void *p1, const void *p2 ) {
    double d1 = *((double*)p1);
    double d2 = *((double*)p2);
    return d1>d2 ? 1 : (d1<d2?-1:0);
}


static double rileva_double_piu_frequente( double *vv, size_t qv ) {
    double v=*vv, vc=*vv;
    size_t i=0, f=0, fMax=0;

    while( i<qv ) {
        while( i<qv&&vv[i]==vc ) { ++i; ++f; }

        if( f>fMax ) { fMax=f; v=vv[i-1]; }

        vc = vv[i];
        f = 0;
    }

    return v;
}


static void rileva_stroke_piu_frequente( ModAttributiSvg *tMod,
                                         TagSvg *t, size_t qt ) {
/* converte tutte le stringhe t[i]._stroke in valori numerici nell'array cc[],
   ordina l'array in senso crescente, quindi rileva il valore che ricorre piu'
   frequentemente, lo riconverte in una stringa e lo colloca in tMod->_stroke */

    uint32_t c, *cc;
    size_t i;

    if( NULL!=(cc=calloc(qt,sizeof(*cc))) ) {
        for( i=0; i<qt; ++i ) {
            cc[i] = -1;

            if( t[i]._stroke && 0!=strcmp(t[i]._stroke,"none") )
                cc[i] = strtoul( t[i]._stroke+1, NULL, 16 );
        }

        qsort( cc, qt, sizeof(*cc), confronta_uint32 );
        c = rileva_uint32_piu_frequente( cc, qt );

        if( ((uint32_t)-1)!=c )
            sprintf( tMod->_stroke, "#%06x", c );

        free( cc );
    }
}


static void rileva_stroke_width_piu_frequente( ModAttributiSvg *tMod,
                                               TagSvg *t, size_t qt ) {
/* converte tutte le stringhe t[i]._stroke_width in valori numerici nell'array
   ww[], ordina l'array in senso crescente, quindi rileva il valore che ricorre
   piu' frequentemente, lo riconverte in una stringa e lo colloca in
   tMod->_stroke_width */

    double w, *ww;
    size_t i;

    if( NULL!=(ww=calloc(qt,sizeof(*ww))) ) {
        for( i=0; i<qt; ++i ) {
            ww[i] = -1.0;

            if( t[i]._stroke && 0!=strcmp(t[i]._stroke_width,"none") )
                ww[i] = strtod( t[i]._stroke_width, NULL );
        }

        qsort( ww, qt, sizeof(*ww), confronta_double );
        w = rileva_double_piu_frequente( ww, qt );

        if( w>0.0 )
            sprintf( tMod->_stroke_width, "%.8lf", w );

        free( ww );
    }
}


static void rileva_stroke_linecap_piu_frequente( ModAttributiSvg *tMod,
                                                 TagSvg *t, size_t qt ) {
    int ff[kTotLinecap]={0};
    size_t i, j;

    for( i=0; i<qt; ++i ) {
        for( j=0; j<kTotLinecap; ++j ) {
            if( t[i]._stroke_linecap ) {
                if( 0==strcmp(t[i]._stroke_linecap,kSLinecap[j]) ) ff[j]++;
            }
        }
    }

    for( j=0, i=0; i<kTotLinecap; ++i )
        j += ff[i]>0;

    if( j>0 ) {
        int fMax=-1, iFMax;

        for( i=0; i<kTotLinecap; ++i )
            if( ff[i]>fMax ) { fMax = ff[i]; iFMax = i; }

        strcpy( tMod->_stroke_linecap, kSLinecap[iFMax] );
    }
}


static void rileva_stroke_linejoin_piu_frequente( ModAttributiSvg *tMod,
                                                  TagSvg *t, size_t qt ) {
    int ff[kTotLinejoin]={0};
    size_t i, j;

    for( i=0; i<qt; ++i ) {
        for( j=0; j<kTotLinejoin; ++j ) {
            if( t[i]._stroke_linejoin ) {
                if( 0==strcmp(t[i]._stroke_linejoin,kSLinejoin[j]) ) ff[j]++;
            }
        }
    }

    for( j=0, i=0; i<kTotLinejoin; ++i )
        j += ff[i]>0;

    if( j>0 ) {
        int fMax=-1, iFMax;

        for( i=0; i<kTotLinejoin; ++i )
            if( ff[i]>fMax ) { fMax = ff[i]; iFMax = i; }

        strcpy( tMod->_stroke_linejoin, kSLinejoin[iFMax] );
    }
}


static void rileva_fill_piu_frequente( ModAttributiSvg *tMod,
                                       TagSvg *t, size_t qt ) {
/* converte tutte le stringhe t[i]._fill in valori numerici nell'array cc[],
   ordina l'array in senso crescente, quindi rileva il valore che ricorre piu'
   frequentemente, lo riconverte in una stringa e lo colloca in tMod->_fill */

    uint32_t c, *cc;
    size_t i;

    if( NULL!=(cc=calloc(qt,sizeof(*cc))) ) {
        for( i=0; i<qt; ++i ) {
            cc[i] = -1;

            if( t[i]._fill && 0!=strcmp(t[i]._fill,"none") )
                cc[i] = strtoul( t[i]._fill+1, NULL, 16 );
        }

        qsort( cc, qt, sizeof(*cc), confronta_uint32 );
        c = rileva_uint32_piu_frequente( cc, qt );

        if( ((uint32_t)-1)!=c )
            sprintf( tMod->_fill, "#%06x", c );

        free( cc );
    }
}


static void rileva_fillrule_piu_frequente( ModAttributiSvg *tMod,
                                           TagSvg *t, size_t qt ) {
    int ff[kTotFillrule]={0};
    size_t i, j;

    for( i=0; i<qt; ++i ) {
        for( j=0; j<kTotFillrule; ++j ) {
            if( t[i]._fill_rule ) {
                if( 0==strcmp(t[i]._fill_rule,kSFillrule[j]) ) ff[j]++;
            }
        }
    }

    for( j=0, i=0; i<kTotFillrule; ++i )
        j += ff[i]>0;

    if( j>0 ) {
        int fMax=-1, iFMax;

        for( i=0; i<kTotFillrule; ++i )
            if( ff[i]>fMax ) { fMax = ff[i]; iFMax = i; }

        strcpy( tMod->_fill_rule, kSFillrule[iFMax] );
    }
}


static void rileva_elementi_piu_frequenti( ModAttributiSvg *tMod,
                                           TagSvg *t, size_t qt ) {
    rileva_stroke_piu_frequente( tMod, t, qt );
    rileva_stroke_width_piu_frequente( tMod, t, qt );
    rileva_stroke_linecap_piu_frequente( tMod, t, qt );
    rileva_stroke_linejoin_piu_frequente( tMod, t, qt );
    rileva_fill_piu_frequente( tMod, t, qt );
    rileva_fillrule_piu_frequente( tMod, t, qt );
}


static int scrivi_stroke_non_std( char *s, char *std, char *ecc ) {
    if( s && *std && ecc && *ecc ) {
        abbrevia_colore( ecc );
        if( strcmp(std,ecc) )
            return sprintf( s, "stroke=\"%s\" ", ecc );
    }

    return 0;
}


static int scrivi_stroke_width_non_std( char *s, char *std, char *ecc ) {
    if( s && *std && ecc && *ecc ) {
        abbrevia_double( ecc );
        if( strcmp(std,ecc) )
            return sprintf( s, "stroke-width=\"%s\" ", ecc );
    }

    return 0;
}


static int scrivi_stroke_linecap_non_std( char *s, char *std, char *ecc ) {
    if( s && *std && ecc && *ecc && strcmp(std,ecc) )
        return sprintf( s, "stroke-linecap=\"%s\" ", ecc );
    return 0;
}


static int scrivi_stroke_linejoin_non_std( char *s, char *std, char *ecc ) {
    if( s && *std && ecc && *ecc && strcmp(std,ecc) )
        return sprintf( s, "stroke-linejoin=\"%s\" ", ecc );
    return 0;
}


static int scrivi_fill_non_std( char *s, char *std, char *ecc ) {
    if( s && *std && ecc && *ecc ) {
        abbrevia_colore( ecc );
        if( strcmp(std,ecc) )
            return sprintf( s, "fill=\"%s\" ", ecc );
    }

    return 0;
}


static int scrivi_fill_rule_non_std( char *s, char *std, char *ecc ) {
    if( s && *std && ecc && *ecc && strcmp(std,ecc) )
        return sprintf( s, "fill-rule=\"%s\" ", ecc );
    return 0;
}


static int scrivi_eccezioni( char *s, ModAttributiSvg *std, TagSvg *ecc ) {
    if( s && std && ecc ) {
        char *p = s;
        p += scrivi_stroke_non_std(
                 p, std->_stroke, ecc->_stroke );
        p += scrivi_stroke_width_non_std(
                 p, std->_stroke_width, ecc->_stroke_width );
        p += scrivi_stroke_linecap_non_std(
                 p, std->_stroke_linecap, ecc->_stroke_linecap );
        p += scrivi_stroke_linejoin_non_std(
                 p, std->_stroke_linejoin, ecc->_stroke_linejoin );
        p += scrivi_fill_non_std(
                 p, std->_fill, ecc->_fill );
        p += scrivi_fill_rule_non_std(
                 p, std->_fill_rule, ecc->_fill_rule );
        return p-s;
    }

    return 0;
}


int scrivi_versione_sintetica( const char *nome_file, SintesiSvg *s ) {
    char *ss = calloc( s->dim_dati_orig+1, sizeof(*ss) );
    FILE *f  = fopen( nome_file, "w+" );

    int ok = ss&&f;

    if( ok ) {
        /* alcuni alias */
        char  **r  = s->r;
        TagSvg *t  = s->t;
        size_t  qt = s->qt;

        int cc   = t->classe; /* cc: classe corrente */
        size_t i = 0;
        char *p = ss;

        /* l'intestazione svg */
        p += sprintf( p, "%s\n%s\n%s\n", r[0], r[1], r[2] );

        /* la sezione dei defs per i path da collegare agli use */
        p += sprintf( p, "<defs>\n" );
        while( i<qt&&kPath!=t[i].tipo ) ++i;
        assegna_id_path( t+i, qt-i );
        p += scrivi_defs( p, t+i, qt-i );
        p += sprintf( p, "</defs>\n" );

        i = 0; /* torna all'inizio della lista dei tags */

        while( i<qt ) {
            ModAttributiSvg tMod = {0}; /* per gli attributi piu' frequenti */
            size_t qEl = conta_tags_classe_corrente( t+i, qt );

            rileva_elementi_piu_frequenti( &tMod, t+i, qEl );

            p += sprintf(p,"<g class=\"%s\"",kSClassi[cc]);
            if( *tMod._stroke )
                p += sprintf(p," stroke=\"%s\"",abbrevia_colore(tMod._stroke));
            if( *tMod._stroke_width )
                p += sprintf(p," stroke-width=\"%s\"",
                             abbrevia_double(tMod._stroke_width));
            if( *tMod._stroke_linecap )
                p += sprintf(p," stroke-linecap=\"%s\"",tMod._stroke_linecap);
            if( *tMod._stroke_linejoin )
                p += sprintf(p," stroke-linejoin=\"%s\"",tMod._stroke_linejoin);
            if( *tMod._fill )
                p += sprintf(p," fill=\"%s\"",tMod._fill);
            if( *tMod._fill_rule )
                p += sprintf(p," fill-rule=\"%s\"",tMod._fill_rule);
            p += sprintf(p,">\n");

            for( ; i<qt&&t[i].classe==cc; ++i ) {
                p += sprintf(p,"%s",kSTipi[t[i].tipo]);

                if( kUse==t[i].tipo&&t[i].id>-1 )
                    p += sprintf(p,"href=\"#%d\" ",t[i].id);

                p += scrivi_eccezioni(p,&tMod,t+i);

                if( t[i]._transform )
                    p += sprintf(p,"transform=\"%s\" ",t[i]._transform);
                if( t[i]._points )
                    p += sprintf(p,"points=\"%s\"/>\n",t[i]._points);
                if( t[i]._d ) {
                    if( kUse!=t[i].tipo )
                        p += sprintf(p,"d=\"%s\"/>\n",t[i]._d);
                    else p += sprintf(p,"/>\n");
                }
            }

            p += sprintf(p,"</g>\n");

            cc = t[i].classe;
        }

        p += sprintf(p,"</svg>\n");

        ok = p-ss == fprintf( f, ss );
    }

    if( f )  fclose( f );
    if( ss ) free( ss );

    return ok;
}
